/*
 * Parts based on arch/powerpc/sysdev/fsl_soc.c:
 *
 * 2006 (c) MontaVista Software, Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/of.h>
#include <linux/i2c.h>
#include <asm/i2c_of.h>

struct i2c_driver_device {
	char    *of_device;
	char    *i2c_driver;
	char    *i2c_type;
};


/* This table is used to map from device tree compat fields to
 * the driver and model used by the i2c board info structures.
 *
 * Add new mappings as needed.
 */

static struct i2c_driver_device i2c_devices[] __initdata = {
	{"ricoh,rs5c372a", "rtc-rs5c372", "rs5c372a",},
	{"ricoh,rs5c372b", "rtc-rs5c372", "rs5c372b",},
	{"ricoh,rv5c386",  "rtc-rs5c372", "rv5c386",},
	{"ricoh,rv5c387a", "rtc-rs5c372", "rv5c387a",},
	{"dallas,ds1307",  "rtc-ds1307",  "ds1307",},
	{"dallas,ds1337",  "rtc-ds1307",  "ds1337",},
	{"dallas,ds1338",  "rtc-ds1307",  "ds1338",},
	{"dallas,ds1339",  "rtc-ds1307",  "ds1339",},
	{"dallas,ds1340",  "rtc-ds1307",  "ds1340",},
	{"stm,m41t00",     "rtc-ds1307",  "m41t00"},
	{"dallas,ds1374",  "rtc-ds1374",  "rtc-ds1374",},
};

int __init of_fill_i2c_info(struct device_node *node,
			    struct i2c_board_info *info)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(i2c_devices); i++) {
		if (!of_device_is_compatible(node, i2c_devices[i].of_device))
			continue;
		if (strlcpy(info->driver_name, i2c_devices[i].i2c_driver,
			    KOBJ_NAME_LEN) >= KOBJ_NAME_LEN ||
		    strlcpy(info->type, i2c_devices[i].i2c_type,
			    I2C_NAME_SIZE) >= I2C_NAME_SIZE)
			return -ENOMEM;
		return 0;
	}
	return -ENODEV;
}
